import numpy as np
from scipy.stats import rankdata
from sklearn.base import BaseEstimator, TransformerMixin

from julia import Julia
Julia(compiled_modules=False)
from julia import Relief as Relief_jl


class ReliefSeq(BaseEstimator, TransformerMixin):
    """sklearn compatible implementation of the ReliefSeq algorithm.

    Reference:
        Brett A. McKinney, Bill C. White, Diane E. Grill, Peter W. Li, Ri-
        chard B. Kennedy, Gregory A. Poland, and Ann L. Oberg. ReliefSeq: a
        gene-wise adaptive-k nearest-neighbor feature selection tool for finding
        gene-gene interactions and main effects in mRNA-Seq gene expression
        data. PloS ONE, 8(12):e81527–e81527, Dec 2013.

    Args:
        n_features_to_select (int): number of features to select from dataset.
        m (int): training data sample size.
        k_min (int): lower ReliefF k parameter interval bound.
        k_max (int): upper ReliefF k parameter interval bound. 
        dist_func (function): function used to measure similarities between samples.
        mode (str): how to compute updated weights from nearest neighbours. 
        Legal values are "k_nearest", "diff" and "exp_rank"
        sig (float): parameter specifying the influence of weights. Ignored if mode = "k_nearest".
        f_type (string): specifies whether the features are continuous or discrete 
        and can either have the value of "continuous" or "discrete".

    Attributes:
        n_features_to_select (int): number of features to select from dataset.
        m (int): training data sample size.
        k_min (int): lower ReliefF k parameter interval bound.
        k_max (int): upper ReliefF k parameter interval bound. 
        dist_func (function): function used to measure similarities between samples.
        mode (str): how to compute updated weights from nearest neighbours. 
        sig (float): parameter specifying the influence of weights. Ignored if mode = "k_nearest".
        f_type (string): continuous or discrete features.
    """
   
    def __init__(self, n_features_to_select=10, m=-1, k_min=5, k_max=10, 
            dist_func=None, mode="k_nearest", sig=1.0, f_type="continuous"):
        self.n_features_to_select = n_features_to_select
        self.m = m 
        self.k_min = k_min
        self.k_max = k_max
        self.dist_func = dist_func
        self.mode = mode
        self.sig = sig
        self.f_type = f_type


    def fit(self, data, target):
        """
        Rank features using ReliefSeq feature selection algorithm

        Args:
            data (numpy.ndarray): matrix of data samples
            target (numpy.ndarray): vector of target values of samples

        Returns:
            (object): reference to self
        """

        # Compute feature weights and rank.
        if self.dist_func is not None:
            self.weights = Relief_jl.reliefseq(data, target, self.m, self.k_min, self.k_max, 
                    self.dist_func, mode=self.mode, sig=self.sig, f_type=self.f_type)
        else:
            self.weights = Relief_jl.reliefseq(data, target, self.m, self.k_min, self.k_max, 
                    mode=self.mode, sig=self.sig, f_type=self.f_type)
        self.rank = rankdata(-self.weights, method='ordinal')
        
        # Return reference to self.
        return self


    def transform(self, data):
        """
        Perform feature selection using computed feature ranks.

        Args:
            data (numpy.ndarray): matrix of data samples on which to perform feature selection.

        Returns:
            (numpy.ndarray): result of performing feature selection.
        """

        # select n_features_to_select best features and return selected features.
        msk = self.rank <= self.n_features_to_select  # Compute mask.
        return data[:, msk]  # Perform feature selection.




import numpy as np
from scipy.stats import rankdata
from sklearn.base import BaseEstimator, TransformerMixin

from julia import Julia
Julia(compiled_modules=False)
from julia import Relief as Relief_jl
from skrelief.relieff import Relieff


class TuRF(BaseEstimator, TransformerMixin):
    """sklearn compatible implementation of the TuRF algorithm.

    Reference:
        Jason H. Moore and Bill C. White. Tuning ReliefF for genome-wide
        genetic analysis. In Elena Marchiori, Jason H. Moore, and Jagath C.
        Rajapakse, editors, Evolutionary Computation,Machine Learning and
        Data Mining in Bioinformatics, pages 166–175. Springer, 2007.

    Args:
        n_features_to_select (int): number of features to select from dataset.
        num_it (int): number of iterations.
        rba (object): feature weighting algorithm wrapped by the VLSRelief algorithm. If equal
        to None, the default ReliefF RBA implemented in Julia is used.

    Attributes:
        n_features_to_select (int): number of features to select from dataset.
        num_it (int): number of iterations.
        _rba (object): feature weighting algorithm wrapped by the TuRF algorithm.
    """
   
    def __init__(self, n_features_to_select=10, num_it=10, rba=None):
        self.n_features_to_select = n_features_to_select
        self.num_it = num_it
        self._rba = rba


    def fit(self, data, target):
        """
        Rank features using TuRF feature selection algorithm

        Args:
            data (numpy.ndarray): matrix of data samples
            target (numpy.ndarray): vector of target values of samples

        Returns:
            (object): reference to self
        """

        # Compute feature weights and rank.
        if self._rba is not None:
            self.weights = Relief_jl.turf(data, target, self.num_it, self.rba_wrap)
        else:
            self.weights = Relief_jl.turf(data, target, self.num_it)
        self.rank = rankdata(-self.weights, method='ordinal')
        
        # Return reference to self.
        return self


    def transform(self, data):
        """
        Perform feature selection using computed feature ranks.

        Args:
            data (numpy.ndarray): matrix of data samples on which to perform feature selection.

        Returns:
            (numpy.ndarray): result of performing feature selection.
        """

        # select n_features_to_select best features and return selected features.
        msk = self.rank <= self.n_features_to_select  # Compute mask.
        return data[:, msk]  # Perform feature selection.




import numpy as np
from scipy.stats import rankdata
from sklearn.base import BaseEstimator, TransformerMixin

from julia import Julia
Julia(compiled_modules=False)
from julia import Relief as Relief_jl


class MultiSURFStar(BaseEstimator, TransformerMixin):
    """sklearn compatible implementation of the MultiSURFStar algorithm.

    Reference:
        Delaney Granizo-Mackenzie and Jason H. Moore. Multiple threshold
        spatially uniform ReliefF for the genetic analysis of complex human
        diseases. In Leonardo Vanneschi, William S. Bush, and Mario Giacobini,
        editors, Evolutionary Computation, Machine Learning and Data Mining
        in Bioinformatics, pages 1–10. Springer, 2013.

    Args:
        n_features_to_select (int): number of features to select from dataset.
        dist_func (function): function used to measure similarities between samples.
        If equal to None, the default implementation of L1 distance in Julia is used.
        f_type (string): specifies whether the features are continuous or discrete 
        and can either have the value of "continuous" or "discrete".

    Attributes:
        n_features_to_select (int): number of features to select from dataset.
        dist_func (function): function used to measure similarities between samples.
        f_type (string): continuous or discrete features.
    """
   
    def __init__(self, n_features_to_select=10, dist_func=None, f_type="continuous"):
        self.n_features_to_select = n_features_to_select
        self.dist_func = dist_func
        self.f_type=f_type


    def fit(self, data, target):
        """
        Rank features using MultiSURFStar feature selection algorithm

        Args:
            data (numpy.ndarray): matrix of data samples
            target (numpy.ndarray): vector of target values of samples

        Returns:
            (object): reference to self
        """

        # Compute feature weights and rank.
        if self.dist_func is not None:
            # If distance function specified.
            self.weights = Relief_jl.multisurfstar(data, target, self.dist_func, f_type=self.f_type)
        else:
            # If distance function not specified, use default L1 distance (implemented in Julia).
            self.weights = Relief_jl.multisurfstar(data, target, f_type=self.f_type)

        self.rank = rankdata(-self.weights, method='ordinal')
        
        # Return reference to self.
        return self


    def transform(self, data):
        """
        Perform feature selection using computed feature ranks.

        Args:
            data (numpy.ndarray): matrix of data samples on which to perform feature selection.

        Returns:
            (numpy.ndarray): result of performing feature selection.
        """

        # select n_features_to_select best features and return selected features.
        msk = self.rank <= self.n_features_to_select  # Compute mask.
        return data[:, msk]  # Perform feature selection.





import numpy as np
from scipy.stats import rankdata
from sklearn.base import BaseEstimator, TransformerMixin

from julia import Julia
Julia(compiled_modules=False)
from julia import Relief as Relief_jl


class Relief(BaseEstimator, TransformerMixin):
    """sklearn compatible implementation of the Relief algorithm.

    Reference:
        Kenji Kira and Larry A. Rendell. The feature selection problem: Tra-
        ditional methods and a new algorithm. In Proceedings of the Tenth
        National Conference on Artificial Intelligence, AAAI’92, pages 129–134.
        AAAI Press, 1992.

    Args:
        n_features_to_select (int): number of features to select from dataset.
        m (int): training data sample size.
        dist_func (function): function used to measure similarities between samples.
        If equal to None, the default implementation of L1 distance in Julia is used.
        f_type (string): specifies whether the features are continuous or discrete 
        and can either have the value of "continuous" or "discrete".

    Attributes:
        n_features_to_select (int): number of features to select from dataset.
        m (int): training data sample size.
        dist_func (function): function used to measure similarities between samples.
        f_type (string): continuous or discrete features.
    """

    def __init__(self, n_features_to_select=10, m=-1, \
            dist_func=None, f_type="continuous"):
        self.n_features_to_select = n_features_to_select
        self.m = m
        self.dist_func = dist_func
        self.f_type = f_type


    def fit(self, data, target):
        """
        Rank features using Relief feature selection algorithm

        Args:
            data (numpy.ndarray): matrix of data samples
            target (numpy.ndarray): vector of target values of samples

        Returns:
            (object): reference to self
        """

        # Compute feature weights and rank.
        if self.dist_func is not None:
            # If distance function specified.
            self.weights = Relief_jl.relief(data, target, self.m, self.dist_func, f_type=self.f_type)
        else:
            # If distance function not specified, use default L1 distance (implemented in Julia).
            self.weights = Relief_jl.relief(data, target, self.m, f_type=self.f_type)
        self.rank = rankdata(-self.weights, method='ordinal')

        # Return reference to self
        return self


    def transform(self, data):
        """
        Perform feature selection using computed feature ranks.

        Args:
            data (numpy.ndarray): matrix of data samples on which to perform feature selection.

        Returns:
            (numpy.ndarray): result of performing feature selection.
        """

        # select n_features_to_select best features and return selected features.
        msk = self.rank <= self.n_features_to_select  # Compute mask.
        return data[:, msk]  # Perform feature selection.




import numpy as np
from scipy.stats import rankdata
from sklearn.base import BaseEstimator, TransformerMixin

from julia import Julia
Julia(compiled_modules=False)
from julia import Relief as Relief_jl
from skrelief.relieff import Relieff


class VLSRelief(BaseEstimator, TransformerMixin):
    """sklearn compatible implementation of the VLSRelief algorithm.

    Reference:
        Margaret Eppstein and Paul Haake. Very large scale ReliefF for genome-
        wide association analysis. In 2008 IEEE Symposium on Computational
        Intelligence in Bioinformatics and Computational Biology, CIBCB ’08,
        2008.

    Args:
        n_features_to_select (int): number of features to select from dataset.
        partition_size (int): size of selected partitions. If equal to -1, use default value computed from data.
        num_partitions_to_select (int): number of partitions to select for each iteration. If equal to
        -1, use default value computed from data.
        num_subsets (int): number of subsets to evaluate.
        rba (object): feature weighting algorithm wrapped by the VLSRelief algorithm. If equal
        to None, the default ReliefF RBA implemented in Julia is used.

    Attributes:
        n_features_to_select (int): number of features to select from dataset.
        partition_size (int): size of selected partitions.
        num_partitions_to_select (int): number of partitions to select for each iteration.
        num_subsets (int): number of subsets to evaluate.
        rba (object): feature weighting algorithm wrapped by the VLSRelief algorithm.
    """
   
    def __init__(self, n_features_to_select=10, partition_size=-1, num_partitions_to_select=-1, num_subsets=20, rba=None):
        self.n_features_to_select = n_features_to_select
        self.partition_size = partition_size
        self.num_partitions_to_select = num_partitions_to_select
        self.num_subsets = num_subsets
        self.rba = rba


    def fit(self, data, target):
        """
        Rank features using VLSRelief feature selection algorithm

        Args:
            data (numpy.ndarray): matrix of data samples
            target (numpy.ndarray): vector of target values of samples

        Returns:
            (object): reference to self
        """

        # Compute feature weights and rank.
        if self.rba is not None:
            # If wrapped RBA specified.
            self.weights = Relief_jl.vlsrelief(data, target, self.partition_size, 
                    self.num_partitions_to_select, self.num_subsets, rba=self.rba)
        else:
            # If wrapped RBA not specified, use default RBA (ReliefF implemented in Julia).
            self.weights = Relief_jl.vlsrelief(data, target, self.partition_size, 
                    self.num_partitions_to_select, self.num_subsets)
        self.rank = rankdata(-self.weights, method='ordinal')
        
        # Return reference to self.
        return self


    def transform(self, data):
        """
        Perform feature selection using computed feature ranks.

        Args:
            data (numpy.ndarray): matrix of data samples on which to perform feature selection.

        Returns:
            (numpy.ndarray): result of performing feature selection.
        """

        # select n_features_to_select best features and return selected features.
        msk = self.rank <= self.n_features_to_select  # Compute mask.
        return data[:, msk]  # Perform feature selection.




import numpy as np
from sklearn.metrics import pairwise_distances
from scipy.stats import rankdata
from sklearn.base import BaseEstimator, TransformerMixin

from julia import Julia
Julia(compiled_modules=False)
from julia import Relief as Relief_jl


class IRelief(BaseEstimator, TransformerMixin):
    """sklearn compatible implementation of the IRelief algorithm.
    
    Reference:
        Yijun Sun and Jian Li. Iterative RELIEF for feature weighting. In ICML
        2006 - Proceedings of the 23rd International Conference on Machine
        Learning, volume 2006, pages 913–920, 2006.
    
    Args:
        n_features_to_select (int): number of features to select from dataset.
        max_iter (int): maximum number of iterations to perform. 
        k_width (int): Kernel width. 
        conv_condition (float): threshold for change in feature weights at which to declare convergence.
        initial_w_div (float): initial value with which to divide the feature weights.
        f_type (string): specifies whether the features are continuous or discrete 
        and can either have the value of "continuous" or "discrete".

    Attributes:
        n_features_to_select (int): number of features to select from dataset.
        max_iter (int): maximum number of iterations to perform. 
        k_width (int): Kernel width. 
        conv_condition (float): threshold for change in feature weights at which to declare convergence.
        initial_w_div (float): initial value with which to divide the feature weights.
        f_type (string): continuous or discrete features.
        _irelief (function): function implementing IRelief written in Julia programming language.

    Author: Jernej Vivod
    """
    
    def __init__(self, n_features_to_select=10, max_iter=100,
            k_width=5, conv_condition=1.0e-12, initial_w_div=1, f_type="continuous"):
        self.n_features_to_select = n_features_to_select  # number of features to select
        self.max_iter = max_iter                          # Maximum number of iterations
        self.k_width = k_width                            # kernel width
        self.conv_condition = conv_condition              # convergence condition
        self.initial_w_div = initial_w_div                # initial weight quotient
        self.f_type = f_type                              # continuous or discrete features


    def fit(self, data, target):
        """
        Rank features using I-Relief feature selection algorithm

        Args:
            data (numpy.ndarray): matrix of data samples
            target (numpy.ndarray): vector of target values of samples

        Returns:
            (object): reference to self
        """

        # Run I-RELIEF feature selection algorithm.
        self.weights = Relief_jl.irelief(data, target, self.max_iter, self.k_width, 
                self.conv_condition, self.initial_w_div, f_type=self.f_type)
        self.rank = rankdata(-self.weights, method='ordinal')

        # Return reference to self.
        return self


    def transform(self, data):
        """
        Perform feature selection using computed feature ranks.

        Args:
            data (numpy.ndarray): matrix of data samples on which to perform feature selection.

        Returns:
            (numpy.ndarray): result of performing feature selection.
        """

        # select n_features_to_select best features and return selected features.
        msk = self.rank <= self.n_features_to_select  # Compute mask.
        return data[:, msk]  # Perform feature selection.




import numpy as np
from scipy.stats import rankdata
from sklearn.base import BaseEstimator, TransformerMixin

from julia import Julia
Julia(compiled_modules=False)
from julia import Relief as Relief_jl


class ECRelieff(BaseEstimator, TransformerMixin):
    """sklearn compatible implementation of the Evaporative Cooling ReliefF algorithm.
    
    Reference:
        B.A. McKinney, D.M. Reif, B.C. White, J.E. Crowe, Jr., J.H. Moore.
        Evaporative cooling feature selection for genotypic data involving interactions.

    Args:
        n_features_to_select (int): number of features to select from dataset.
        m (int): training data sample size.
        k (int): number of nearest neighbours to find (for each class).
        dist_func (function): function used to measure similarities between samples.
        If equal to None, the default implementation of L1 distance in Julia is used.
        mode (string): specifies which type of weights update to perform and can either have the value of "k_nearest", "diff" or "exp_rank" (see reference paper). 
        sig (float): kernel width used when mode has the value of "exp_rank".
        f_type (string): specifies whether the features are continuous or discrete 
        and can either have the value of "continuous" or "discrete".

    Attributes:
        n_features_to_select (int): number of features to select from dataset.
        m (int): training data sample size.
        k (int): number of nearest neighbours to find (for each class).
        dist_func (function): function used to measure similarities between samples.
        mode (string): which type of weights update to perform.
        sig (float): kernel width (when mode has the value of "exp_rank").
        f_type (string): continuous or discrete features.

    Author: Jernej Vivod
    """
  

    def __init__(self, n_features_to_select=10, m=-1, k=5, dist_func=None, mode="k_nearest", sig=1.0, f_type="continuous"):
        self.n_features_to_select = n_features_to_select
        self.m = m                                        
        self.k = k                                        
        self.dist_func = dist_func
        self.mode = mode
        self.sig = sig
        self.f_type = f_type


    def fit(self, data, target):
        """
        Rank features using Evaporative Cooling Relieff feature selection algorithm

        Args:
            data (numpy.ndarray): matrix of data samples
            target (numpy.ndarray): vector of target values of samples

        Returns:
            (object): reference to self
        """
        
        # Rank features.
        if self.dist_func is not None:
            # If distance function specified.
            self.rank = Relief_jl.ecrelieff(data, target, self.m, self.k, self.dist_func, 
                    mode=self.mode, sig=self.sig, f_type=self.f_type)
        else:
            # If distance function not specified, use default L1 distance (implemented in Julia).
            self.rank = Relief_jl.ecrelieff(data, target, self.m, self.k, 
                    mode=self.mode, sig=self.sig, f_type=self.f_type)

        return self


    def transform(self, data):
        """
        Perform feature selection using computed feature ranks.

        Args:
            data (numpy.ndarray): matrix of data samples on which to perform feature selection.

        Returns:
            (numpy.ndarray): result of performing feature selection.
        """
       
       # select n_features_to_select best features and return selected features.
        msk = self.rank <= self.n_features_to_select  # Compute mask.
        return data[:, msk]  # Perform feature selection.




from setuptools import setup

setup(name='skrelief',
      version='0.1',
      description='scikit-learn compatible interface to a number of Relief-based feature selection algorithms',
      url='https://github.com/jernejvivod/skrelief',
      author='Jernej Vivod',
      author_email='vivod.jernej@gmail.com',
      license='MIT',
      packages=['skrelief'],
      install_requires=[
          'numpy',
          'scipy',
          'sklearn',
          'julia',
      ],
      classifiers=[
          'Intended Audience :: Developers',
          'Intended Audience :: Information Technology',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Topic :: Utilities'
      ],
      keywords=['data mining', 'feature selection', 'feature evaluation', 'machine learning', 'data analysis', 'artificial intelligence', 'data science'],
      include_package_data=True,
      zip_safe=False)



import numpy as np
from scipy.stats import rankdata
from sklearn.base import BaseEstimator, TransformerMixin
import warnings

from julia import Julia
Julia(compiled_modules=False)
from julia import Relief as Relief_jl


class ReliefMSS(BaseEstimator, TransformerMixin):
    """sklearn compatible implementation of the ReliefMSS algorithm

    Reference:
        Salim Chikhi and Sadek Benhammada. ReliefMSS: A variation on a
        feature ranking ReliefF algorithm. IJBIDM, 4:375–390, 2009.

    Args:
        n_features_to_select (int): number of features to select from dataset.
        m (int): training data sample size.
        k (int): number of nearest neighbours to find (for each class).
        dist_func (function): function used to measure similarities between samples.
        If equal to None, the default implementation of L1 distance in Julia is used.
        f_type (string): specifies whether the features are continuous or discrete 
        and can either have the value of "continuous" or "discrete".

    Attributes:
        n_features_to_select (int): number of features to select from dataset.
        m (int): training data sample size.
        k (int): number of nearest neighbours to find (for each class).
        dist_func (function): function used to measure similarities between samples.
        f_type (string): continuous or discrete features.
    """
   
    def __init__(self, n_features_to_select=10, m=-1, k=5, dist_func=None, f_type="continuous"):
        self.n_features_to_select = n_features_to_select
        self.m = m 
        self.k = k
        self.dist_func = dist_func
        self.f_type = f_type


    def fit(self, data, target):
        """
        Rank features using ReliefMSS feature selection algorithm

        Args:
            data (numpy.ndarray): matrix of data samples
            target (numpy.ndarray): vector of target values of samples

        Returns:
            (object): reference to self
        """

        # Get number of instances with class that has minimum number of instances.
        _, instances_by_class = np.unique(target, return_counts=True)
        min_instances = np.min(instances_by_class) - 1
       
        # If class with minimal number of examples (minus one) has less than k examples, issue warning
        # that parameter k was reduced.
        if min_instances < self.k:
            warnings.warn("Parameter k was reduced to {0} because one of the classes " \
                "does not have {1} instances associated with it.".format(min_instances, self.k), Warning)

        # Compute feature weights and rank.
        if self.dist_func is not None:
            # If distance function specified.
            self.weights = Relief_jl.reliefmss(data, target, self.m, int(min(self.k, min_instances)), self.dist_func, f_type=self.f_type)
        else:
            # If distance function not specified, use default L1 distance (implemented in Julia).
            self.weights = Relief_jl.reliefmss(data, target, self.m, int(min(self.k, min_instances)), f_type=self.f_type)
        self.rank = rankdata(-self.weights, method='ordinal')
        
        # Return reference to self.
        return self


    def transform(self, data):
        """
        Perform feature selection using computed feature ranks.

        Args:
            data (numpy.ndarray): matrix of data samples on which to perform feature selection.

        Returns:
            (numpy.ndarray): result of performing feature selection.
        """

        # select n_features_to_select best features and return selected features.
        msk = self.rank <= self.n_features_to_select  # Compute mask.
        return data[:, msk]  # Perform feature selection.

